import csv
from copy import deepcopy
import numpy as np
import scipy.stats as ss
import pystan
import matplotlib.pyplot as plt
import seaborn as sb

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

fin_nm = input('Input file (*.csv) = ')
with open(fin_nm, 'r') as f:
    Data_in = [v for v in csv.reader(f)]

Data_in = np.array(Data_in)
Rs = [int(v) for v in Data_in.T[1][1:]]
As = [float(v) for v in Data_in.T[2][1:]]

print(len(Rs), len(As))
N = len(Rs)
Rs = np.array(Rs)
As = np.array(As)
Raw_As = deepcopy(As)
As_mean = As.mean()
As_sd = As.std()
As = (As - As_mean) / As_sd

#       Stanスクリプトのコンパイル
sm = pystan.StanModel(file = 'reg_age.stan')

#        MCMCサンプリング
fit = sm.sampling(data = {'N':N, 'Rs': Rs, 'As': As},
                  pars = ['a', 'b'], iter = 10_000, n_jobs = 1)
print(fit)

#
#      modeの推定
#
def est_mode(samples, a = 0.05, n_points = 10_000):
    import numpy as np
    import scipy.stats as ss
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]   
    return MAP_Est
  
a_mode = est_mode(fit['a'])
b_mode = est_mode(fit['b'])

#     パラメータaの事後分布のグラフ
sb.kdeplot(fit['a'], label = f'mode = {a_mode:.2f}')
plt.xlabel('a', fontsize = 14)
plt.yticks([])
plt.title('aの事後分布', fontsize = 16)
plt.legend(fontsize = 14)
plt.savefig('Fig_a.png')
plt.show()
#     パラメータbの事後分布のグラフ
sb.kdeplot(fit['b'], label = f'mode = {b_mode:.2f}')
plt.xlabel('b', fontsize = 14)
plt.yticks([])
plt.title('bの事後分布', fontsize = 16)
plt.legend(fontsize = 14)
plt.savefig('Fig_b.png')
plt.show()
#
#        値のカテゴリ化
#    0:[-inf, 29), 1:[30, 39), 2:[40, 49), 3:[50, +inf)
#
def check_cat(v):
    v = int((v - 20) / 10)
    if v > 3:
        v = 3
    return v

Ones = np.zeros(4)    #   各カテゴリにおける判断１の数
Boths = np.zeros(4)   #   各カテゴリにおける判断１および０の数
for r, a in zip(Rs, Raw_As):
    icat = check_cat(a)
    Boths[icat] += 1
    if r == 1:
        Ones[icat] += 1
    
print(Ones)
print(Boths)

#     Ｘ軸上のグリッド点100個
x_coord = np.linspace(As.min(), As.max(), 100)

def Pd(u):
    return 1.0 / (1.0 + np.exp(-u))

def Pc(x):
    return Pd(x) + (1.0 - Pd(x)) / 3

y_Pc_m = Pc(a_mode + b_mode*x_coord)
y_Pd_m = Pd(a_mode + b_mode*x_coord)
plt.plot(x_coord, y_Pc_m, label = 'Pc')
plt.plot(x_coord, y_Pd_m, label = 'Pd')
#
#     各カテゴリ区間におけるデータの正答率
#
data_x = [(v - As_mean)/As_sd for v in [25, 35, 45, 55] ]
data_y = [Ones[i]/Boths[i] for i in range(4)]
plt.plot(data_x, data_y, 'o', label = 'Data')

x_vs = [(v - As_mean)/As_sd for v in [20, 30, 40, 50, 60]]
x_lbls = [f'{v}' for v in [20, 30, 40, 50, 60]]
plt.xticks(x_vs, x_lbls)
plt.xlabel('独立変数', fontsize = 14)
plt.legend(fontsize = 14)
plt.ylabel('確率', fontsize = 14)
plt.title('３点識別試験', fontsize = 16)
plt.savefig('FigRegAge.png')
plt.show()
